import setuptools


setuptools.setup(
    name = "visioncam",
    version = "0.0.1",
    license='MIT',
    author = 'Ahmed Tolba',
    author_email = 'atolba210@gmail.com',
    description = 'XAI FrameWork for computer vision tasks',
    packages = setuptools.find_packages(),

    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
   
   

    )


